@@ -103,6 +103,19 @@ setAs("ntTMatrix", "ntCMatrix",
 ##  (i , j) \in
 ##  (numeric, logical, character, missing) x (numeric, log., char., miss.)
 
+##' a simplified "subset" of  intI() below
+int2i <- function(i, n) {
+    if(any(i < 0L)) {
+	if(any(i > 0L))
+	    stop("you cannot mix negative and positive indices")
+	seq_len(n)[i]
+    } else {
+	if(length(i) && max(i, na.rm=TRUE) > n)
+	    stop(gettextf("index larger than maximal %d", n), domain=NA)
+	if(any(z <- i == 0)) i <- i[!z]
+	i
+    }
+}
 
 intI <- function(i, n, dn, give.dn = TRUE)
 {
@@ -115,7 +128,7 @@ intI <- function(i, n, dn, give.dn = TRUE)
     ## ----------------------------------------------------------------------
     ## Author: Martin Maechler, Date: 23 Apr 2007
 
-    has.dn <- !is.null(dn)
+    has.dn <- !is.null.DN(dn)
     DN <- has.dn && give.dn
     if(is(i, "numeric")) {
 	storage.mode(i) <- "integer"
@@ -127,7 +140,7 @@ intI <- function(i, n, dn, give.dn = TRUE)
 	    i0 <- (0:(n - 1L))[i]
 	} else {
 	    if(length(i) && max(i, na.rm=TRUE) > n)
-		stop("index larger than maximal ",n)
+		stop(gettextf("index larger than maximal %d", n), domain=NA)
 	    if(any(z <- i == 0)) i <- i[!z]
 	    i0 <- i - 1L		# transform to 0-indexing
 	}
@@ -136,7 +149,7 @@ intI <- function(i, n, dn, give.dn = TRUE)
     else if (is(i, "logical")) {
 	if(length(i) > n)
 	    stop(gettextf("logical subscript too long (%d, should be %d)",
-			  length(i), n))
+			  length(i), n), domain=NA)
 	i0 <- (0:(n - 1L))[i]
 	if(DN) dn <- dn[i]
     } else { ## character
@@ -297,12 +310,12 @@ replTmat <- function (x, i, j, ..., value)
     spV <- extends(clDv, "sparseVector")
     ## own version of all0() that works both for sparseVector and atomic vectors:
     .all0 <- function(v) if(spV) length(v@i) == 0 else all0(v)
-    delayedAssign("value.is.logical",
-                  if(spV) {
+    delayedAssign("value.not.logical",
+                  !(if(spV) {
                       extends(clDv, "lsparseVector") || extends(clDv, "nsparseVector")
                   } else {
                       is.logical(value) || is.logical(as.vector(value))
-                  })
+                  }))
     na <- nargs()
     if(na == 3) { ## i = vector (but *not* 2-col) indexing"  M[i] <- v
 	Matrix.msg("diagnosing replTmat(x,i,j,v): nargs()= 3; ",
@@ -328,28 +341,39 @@ replTmat <- function (x, i, j, ..., value)
 	    x.i <- .Call(m_encodeInd2, x@i, x@j, di=di, FALSE)
 	}
 
-	if(is.logical(i)) { # full-size logical indexing
-	    n <- prod(di)
+        n <- prod(di)
+	i <- if(is.logical(i)) { # full-size logical indexing
 	    if(n) {
-		if(length(i) < n) i <- rep(i, length.out = n)
-		i <- (0:(n-1))[i] # -> 0-based index vector as well {maybe LARGE!}
-	    } else i <- integer(0)
-	} else i <- as.integer(i) - 1L ## 0-based indices
+		if(length(i) < n) i <- rep_len(i, n)
+		(0:(n-1))[i] # -> 0-based index vector as well {maybe LARGE!}
+	    } else integer(0)
+	} else {
+	    ## also works with *negative* indices etc:
+	    int2i(as.integer(i), n) - 1L ## 0-based indices [to match m_encodeInd2()]
+	}
 
         clx <- class(x)
         clDx <- getClassDef(clx) # extends(), is() etc all use the class definition
         has.x <- "x" %in% slotNames(clDx) # === slotNames(x)
 	if(!has.x && # <==> "n.TMatrix"
-	   ((iNA <- any(is.na(value))) || !value.is.logical))
-	    warning(gettextf("x[.] <- val: x is \"%s\", val not in {TRUE, FALSE} is coerced%s.",
-			     clx, if(iNA) " NA |--> TRUE" else ""))
+	   ((iNA <- any(ina <- is.na(value))) || value.not.logical)) {
+            if(value.not.logical) value <- as.logical(value)
+	    if(iNA) {
+		value[ina] <- TRUE
+		warning(
+		    gettextf("x[.] <- val: x is %s, val not in {TRUE, FALSE} is coerced; NA |--> TRUE.",
+			     dQuote(clx)), domain=NA)
+	    }
+	    else warning(
+		    gettextf("x[.] <- val: x is %s, val not in {TRUE, FALSE} is coerced.",
+			     dQuote(clx)), domain=NA)
+	}
 
 	## now have 0-based indices   x.i (entries) and	 i (new entries)
 
 	## the simplest case:
 	if(.all0(value)) { ## just drop the non-zero entries
-	    sel <- is.na(match(x.i, i))
-	    if(any(!sel)) { ## non-zero there
+	    if(!all(sel <- is.na(match(x.i, i)))) { ## non-zero there
 		x@i <- x@i[sel]
 		x@j <- x@j[sel]
 		if(has.x)
@@ -364,24 +388,49 @@ replTmat <- function (x, i, j, ..., value)
 	if(length(value) != m) { ## use recycling rules
 	    if(m %% length(value) != 0)
 		warning("number of items to replace is not a multiple of replacement length")
-	    value <- rep(value, length.out = m)
+	    value <- rep_len(value, m)
 	}
 
-	## matching existing non-zeros and new entries
-	isE <- !is.na(mi <- match(i, x.i)) ## use  which(isE) , mi[isE]
+        ## With duplicated entries i, only use the last ones!
+        if(id <- anyDuplicated(i, fromLast=TRUE)) {
+            i <- i[-id]
+            value <- value[-id]
+            if(any(id <- duplicated(i, fromLast=TRUE))) {
+                nd <- -which(id)
+                i <- i[nd]
+                value <- value[nd]
+            }
+        }
+
+	## matching existing non-zeros and new entries; isE := "is Existing"
+	##  isE <- i %in% x.i;  mi <- {matching i's}
+        isE <- !is.na(mi <- match(i, x.i))
+        ## => mi[isE] entries in (i,j,x) to be set to new value[]s
+
 	## 1) Change the matching non-zero entries
 	if(has.x)
 	    x@x[mi[isE]] <- value[isE]
+        else if(any0(value[isE])) { ## "n.TMatrix" : remove (i,j) where value is FALSE
+            get0 <- !value[isE] ## x[i,j] is TRUE, should become FALSE
+            i.rm <- - mi[isE][get0]
+            x@i <- x@i[i.rm]
+            x@j <- x@j[i.rm]
+        }
 	## 2) add the new non-zero entries
 	i <- i[!isE]
+	xv <- value[!isE]
+	if(has.x) {
+            x@x <- c(x@x, xv)
+	} else { # n.TMatrix : assign (i,j) only where value is TRUE:
+	    i <- i[xv]
+	}
 	x@i <- c(x@i, i %%  nr)
 	x@j <- c(x@j, i %/% nr)
-	if(has.x)
-	    x@x <- c(x@x, value[!isE])
 	if(extends(clDx, "compMatrix") && length(x@factors)) # drop cashed ones
 	    x@factors <- list()
 	return(x)
-    }
+    } ## {nargs = 3;  x[ii] <- value }
+
     ## nargs() == 4 :  x[i,j] <- value
     ## --------------------------------------------------------------------------
     lenV <- length(value)
@@ -412,7 +461,7 @@ replTmat <- function (x, i, j, ..., value)
         keep <- !dup
         i1 <- i1[keep]
         ## keep is "internally" recycled below {and that's important: it is dense!}
-        lenV <- length(value <- rep(value, length.out = lenRepl)[keep])
+	lenV <- length(value <- rep_len(value, lenRepl)[keep])
         dind[1] <- length(i1)
         lenRepl <- prod(dind)
     }
@@ -421,12 +470,12 @@ replTmat <- function (x, i, j, ..., value)
         ## The following is correct, but  rep(keep,..) can be *HUGE*
         ## keep <- !dup
         ## i2 <- i2[keep]
-        ## lenV <- length(value <- rep(value, length.out = lenRepl)[rep(keep, each=dind[1])])
+	## lenV <- length(value <- rep_len(value, lenRepl)[rep(keep, each=dind[1])])
         ## solution: sv[-i] is efficient for sparseVector:
         i2 <- i2[- iDup]
         nr <- dind[1]
         iDup <- rep((iDup - 1)*nr, each=nr) + seq_len(nr)
-        lenV <- length(value <- rep(value, length.out = lenRepl)[-iDup])
+	lenV <- length(value <- rep_len(value, lenRepl)[-iDup])
         dind[2] <- length(i2)
         lenRepl <- prod(dind)
     }
@@ -496,9 +545,13 @@ replTmat <- function (x, i, j, ..., value)
     ## now have  lenV <= lenRepl
 
     if(!has.x && # <==> "n.TMatrix"
-       ((iNA <- any(is.na(value))) || !value.is.logical))
-	warning(gettextf("x[.,.] <- val: x is \"%s\", val not in {TRUE, FALSE} is coerced%s.",
-			 clx, if(iNA) " NA |--> TRUE" else ""))
+       ((iNA <- any(is.na(value))) || value.not.logical))
+	warning(if(iNA)
+		gettextf("x[.,.] <- val: x is %s, val not in {TRUE, FALSE} is coerced NA |--> TRUE.",
+			 dQuote(clx))
+		else
+		gettextf("x[.,.] <- val: x is %s, val not in {TRUE, FALSE} is coerced.",
+			 dQuote(clx)), domain=NA)
 
     ## another simple, typical case:
     if(lenRepl == 1) {
@@ -524,7 +577,8 @@ replTmat <- function (x, i, j, ..., value)
 ## if(identical(Sys.getenv("USER"),"maechler")
 ##    if(lenRepl > 2) { # __________ ___ JUST for testing! _______________
 	if(is.null(v <- getOption("Matrix.quiet")) || !v)
-	    message(gettextf("x[.,.] <- val : x being coerced from Tsparse* to CsparseMatrix"))
+	    message(gettextf("x[.,.] <- val : x being coerced from Tsparse* to CsparseMatrix"),
+		    domain = NA)
 	return(replCmat4(as(x,"CsparseMatrix"), i1, i2, iMi=iMi, jMi=jMi,
 			 value = if(spV) value else as(value, "sparseVector"),
 			 spV = TRUE))
@@ -533,7 +587,7 @@ replTmat <- function (x, i, j, ..., value)
     ##     if(r.sym) # value already adjusted, see above
     ##        lenRepl <- length(value) # shorter (since only "triangle")
     if(!r.sym && lenV < lenRepl)
-	value <- rep(value, length.out = lenRepl)
+	value <- rep_len(value, lenRepl)
 
     ## now:  length(value) == lenRepl  {but value is sparseVector if it's "long" !}
 
@@ -619,7 +673,7 @@ replTmat <- function (x, i, j, ..., value)
 {
     nA <- nargs()
     if(nA != 3)
-	stop(gettextf("nargs() = %d should never happen; please report.", nA))
+	stop(gettextf("nargs() = %d should never happen; please report.", nA), domain=NA)
 
     ## else: nA == 3  i.e.,  M [ cbind(ii,jj) ] <- value or M [ Lmat ] <- value
     if(is.logical(i)) {
@@ -632,6 +686,10 @@ replTmat <- function (x, i, j, ..., value)
 	i <- which(as(i, if(extends(cli, "sparseMatrix")) "sparseVector" else "vector"))
 	## x[i] <- value ; return(x)
 	return(`[<-`(x,i, value=value))
+    } else if(extends(cli, "Matrix")) { # "dMatrix" or "iMatrix"
+	if(ncol(i) != 2)
+	    stop("such indexing must be by logical or 2-column numeric matrix")
+	i <- as(i, "matrix")
     } else if(!is.numeric(i) || ncol(i) != 2)
 	stop("such indexing must be by logical or 2-column numeric matrix")
     if(!is.integer(i)) storage.mode(i) <- "integer"
@@ -653,7 +711,7 @@ replTmat <- function (x, i, j, ..., value)
     if(length(value) != m) { ## use recycling rules
 	if(m %% length(value) != 0)
 	    warning("number of items to replace is not a multiple of replacement length")
-	value <- rep(value, length = m)
+	value <- rep_len(value, m)
     }
     clx <- class(x)
     clDx <- getClassDef(clx) # extends() , is() etc all use the class definition
@@ -664,8 +722,8 @@ replTmat <- function (x, i, j, ..., value)
     nc <- di[2]
     i1 <- i[,1]
     i2 <- i[,2]
-    if(any(i1 > nr)) stop("row indices must be <= nrow(.) which is ", nr)
-    if(any(i2 > nc)) stop("column indices must be <= ncol(.) which is ", nc)
+    if(any(i1 > nr)) stop(gettextf("row indices must be <= nrow(.) which is %d", nr), domain=NA)
+    if(any(i2 > nc)) stop(gettextf("column indices must be <= ncol(.) which is %d", nc), domain=NA)
 
     ## Tmatrix maybe non-unique, have an entry split into a sum of several ones:
     if(is_duplicatedT(x, di = di))
@@ -721,12 +779,16 @@ replTmat <- function (x, i, j, ..., value)
     }
 
     ii.v <- .Call(m_encodeInd, i - 1L, di, checkBounds = TRUE)# 0-indexing
-    if(any(d <- duplicated(rev(ii.v)))) { # reverse: "last" duplicated FALSE
-	warning("duplicate ij-entries in 'Matrix[ ij ] <- value'; using last")
-	nd <- !rev(d)
-	## i  <- i    [nd, , drop=FALSE]
-	ii.v  <- ii.v [nd]
-	value <- value[nd]
+    if(id <- anyDuplicated(ii.v, fromLast=TRUE)) {
+        Matrix.msg("duplicate ij-entries in 'Matrix[ ij ] <- value'; using last",
+                   .M.level = 1)
+        ii.v  <- ii.v [-id]
+	value <- value[-id]
+        if(any(id <- duplicated(ii.v, fromLast=TRUE))) {
+            nd <- -which(id)
+            ii.v  <- ii.v [nd]
+            value <- value[nd]
+        }
     }
     ii.x <- .Call(m_encodeInd2, x@i, x@j, di, FALSE)
     m1 <- match(ii.v, ii.x)
@@ -779,6 +841,9 @@ setReplaceMethod("[", signature(x = "TsparseMatrix", i = "index", j = "index",
 setReplaceMethod("[", signature(x = "TsparseMatrix", i = "matrix", j = "missing",
 				value = "replValue"),
 		 .TM.repl.i.mat)
+setReplaceMethod("[", signature(x = "TsparseMatrix", i = "Matrix", j = "missing",
+				value = "replValue"),
+		 .TM.repl.i.mat)
 
 
 ### When the RHS 'value' is  a sparseVector, now can use  replTmat  as well
