@@ -60,6 +60,7 @@ int R_cholmod_start(CHM_CM Common);
 int R_cholmod_l_start(CHM_CM Common);
 void R_cholmod_error(int status, const char *file, int line, const char *message);
 
+SEXP get_SuiteSparse_version();
 SEXP chm_factor_to_SEXP(CHM_FR f, int dofree);
 SEXP chm_sparse_to_SEXP(CHM_SP a, int dofree, int uploT, int Rkind,
 			const char *diag, SEXP dn);
@@ -68,6 +69,8 @@ SEXP chm_triplet_to_SEXP(CHM_TR a, int dofree, int uploT, int Rkind,
 SEXP chm_dense_to_SEXP(CHM_DN a, int dofree, int Rkind, SEXP dn);
 /* 		       int uploST, char *diag, SEXP dn); */
 SEXP chm_dense_to_matrix(CHM_DN a, int dofree, SEXP dn);
+SEXP chm_dense_to_vector(CHM_DN a, int dofree);
+
 
 void chm_diagN2U(CHM_SP chx, int uploT, Rboolean do_realloc);
 
